<?php
/**
 * appRain v 0.1.x
 *
 * LICENSE
 *
 * This source file is subject to the MIT license that is bundled
 * with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.opensource.org/licenses/mit-license.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@apprain.com so we can send you a copy immediately.
 *
 * @copyright  Copyright (c) 2010 appRain, Team. (http://www.apprain.com)
 * @license    http://www.opensource.org/licenses/mit-license.php MIT license
 *
 * HELP
 *
 * Official Website
 * http://www.apprain.com/
 *
 * Download Link
 * http://www.apprain.com/download
 *
 * Documents Link
 * http ://www.apprain.com/docs
 */


/**
 * Home page
 */
class homeController extends appRain_Base_Core
{
    // Controller Name
    public $name = "Home";
    public $dispatch =  Array(
								'preDispatchExclude'=>array(),
								'postDispatchExclude'=>array());

    /**
     * Function Call before Page action method each time
     */
    public function __preDispatch()
    {
        if(App::Membermanager()->isProfileUserLoggedin())
        {
            App::Helper('Config')->transfer($this->baseurl('/profile-user-information'),
                                            $this->__("Profile user is logged in."));
        }
    }

    /**
     * Function Call before Page action method each time
     */
    public function __postDispatch()
    {
    }

    /**
	 * Render Home page
     * We have configure this page from
     * URI_Manager >> Boot_Router
     * to be Render as a starting page of the project
     */
    public function indexAction($id=null)
    {
		// Register Addons
        $this->addons = array('photo_viewer','appslide');

		// Fetch data from static page manager and
		// set Page Meta Information.
		$homepage = $this->staticPageNameToMetaInfo('HomePage');

		// Section run for AJAX request
		// so we have set layout to empty
		if(isset($id))
        {
            $this->layout="empty";
            $data  = $this->InformationSet()->findById($id);
            echo $data['description'];
            exit;
        }

		// Fetch Home Slide Data and assigned to
		// Tempate.
		$home_press = App::InformationSet('home-press')->findAll("1 ORDER BY Information.generic ASC",Array(Array("status"=>"Active")));
		$this->set("pressdata",$home_press);

		// Fetch homepage synopsis section data
		// and assign to Template
		$home_synopsis = App::InformationSet('home-synopsis')->findAll("1 ORDER BY Information.generic ASC",Array(Array("status"=>"Active")));
		$this->set("home_synopsis",$home_synopsis);

        // Set value to template
        $this->set("selected","home");
    }

	/**
	 * Contact us
	 *
	 */
	public function contactusAction()
	{
		// Attach all necessary addons
		$this->addons = Array('defaultvalues','validation','ajaxsubmit');
		// Set meta information and fetch data
		// from Page Manager
		$staticpage = $this->staticPageNameToMetaInfo('contact-us');

		// This section run if there is any
		// submit by POST method. Post data saved in
		// $this->data if we follow field name like
		// "data[Comments][email]
		if(!empty($this->data))
		{
			$this->layout = 'empty';

			try
			{
				// Read capacha session value
				$capacha = App::Module('Session')->read('capacha');
				// Compare capacha value and throw exception
				// for Invalid Input
				if($capacha['contactus'] != $this->data['Contact']['capacha'])
				{
					throw new AppException($this->__("Please fillin the text display left side image correctly."));
				}

				// Prepare Email template Parameters
				$params = Array(
									'FirstName'=>$this->data['Contact']['f_name'],
									'LastName'=>$this->data['Contact']['l_name'],
									'Email'=>$this->data['Contact']['email'],
									'Subject'=>$this->data['Contact']['subject'],
									'Message'=>$this->data['Contact']['message']
								);

				// Prepare tempate and send to User.
				// We can set a specific reciver by a magic
				// method setTo(Array('email','name')) also
				// sender like setFrom(Array('email','name'))
				App::Helper('EmailTemplate')->setParameters($params)
											->prepare('ContactUs',true);

				// Throw final exception with Success status
				// We also do that by displaying the message and status
				// in JSON format without throwing Exception.
				$status = 'Success';
				throw new AppException($this->__("Thank you, Message sent successfully."));
			}
			catch(AppException $e)
			{
				// Simple pack the data in JSON format and
				// display the outpost for AJAX module
				echo App::Load("Module/Cryptography")->jsonEncode(array(
																	"_status" =>(isset($status) ? $status : 'Error'),
																	"_message"=>$e->getMessage()
															  ));
			}
		}

		// Set common values in template
		$this->set("staticpage",$staticpage);
		$this->set("selected","contactus");
		$this->set("section_title","Contact Us");
	}

	/**
	 * Create search result based on Definitions
	 * and call back functions.
	 */
	public function searchAction($srcstr = null, $page = 1)
	{
		// Attach Addons and Set meta information
		$this->addons = Array('defaultvalues');
		$staticpage = $this->staticPageNameToMetaInfo('search');

		// Fetch all search data definition in defintion
		// for Information Set and Category set
		$srcstr = isset($this->post['ss']) ? $this->post['ss'] : $srcstr;
		$srcData = App::Helper("Search")
					  ->setSmartPaging(true)
					  ->setPage($page)
					  ->setLimit(5)
					  ->setHLink($this->baseurl("/search/{$srcstr}"))
					  ->Search($srcstr);

		// Overwrite page title
		$this->page_title = "{$srcstr} {$this->page_title}";
		// Set Search data
		$this->set('srcstr', $srcstr);
		$this->set('srcarr', $srcData);
		// Set Common variables
		$this->set("section_title","Search Reasult");
		$this->set("selected","search");
	}
}
